
/********************************
bpj_load.js
The data model file of the visualization
Used for parsing the xml, setting the database, and as an interface to the visual side
Written by Nir Eitan, Wiezmann Is., 2010
***********************************/


var _doc;
var _bthreads_num;
var _syncpoints_num;
var _bthreads_array;
var _syncthreads_array;
var _syncpoints_array;
var _start_point;
var _bthread_type_counters = [0,0,0,0];
var _sync_in_page;
var _type_filter = [true ,true, true, true];  //true if a bthread type should be shown
var _class_based = false; //if only classes are shown
var _event_grouped = false; //to group by events
var _filtered_bthread_list;

// The style is the only thing differentiating BPJ from PLAYGO right now.
// The only difference now is the [I] block, which as understood,
// should be implemented in BPJ like it is implemented in PLAYGO.
// For now, BPJ outputs with:
//     <EventTrace style="BPJ">
// and playgo, with:
//     <EventTrace style="playgo">
var _eventtrace_style = false;

/**
 modifies the list of "active" bthreads (after filtering)
**/
function set_filtered_bthread_list (){
	var filtered_array = new Array();
	for (i=1;i<_bthreads_array.length;i++)
	{
		var bthr = get_bthread(i);
		if (is_filtered(bthr)) continue;
		if (_class_based) {
			bthr.name = "::"+bthr.bclass;
			var flag=0;
			for (var j=0;j<filtered_array.length; j++)
			{
				if (filtered_array[j].bclass == bthr.bclass)
				{
					
						
					//add it to the previous bthread
					filtered_array[j].combine(bthr);
					flag=1;
					break;
				}
			}
			if (flag==0)
				filtered_array.push(bthr);
			
		} else
			filtered_array.push(bthr);
	}
	_filtered_bthread_list =  filtered_array;
}
function get_filtered_bthread_list () {
	return _filtered_bthread_list;
}
function set_class_grouping (bool) {
	_class_based = bool;
}
function set_event_grouping (bool) {
	_event_grouped = bool;
}
function set_sync_num_in_page(num) {
	_sync_in_page = num;
}
function get_sync_num_in_page() {
	return _sync_in_page;
}

function set_bthread(node) {
		id = node.getAttribute('id');
		var bth = get_bthread_by_id(id)
		if (bth!=0) 
			return bth;
		else
			return new bthread(node);
}
	
	
	function get_bthread_by_id(id) {
		for (var i=1;i<=_bthreads_array.length-1;i++) 
			if (_bthreads_array[i].id == id)
				return _bthreads_array[i];
		return 0;
	}
	
	function get_bthread_by_name(name) {
		for (var i=1;i<=_bthreads_array.length-1;i++) 
			if (_bthreads_array[i].name == name)
				return _bthreads_array[i];
		return 0;
	}
	
	function get_bthread(bind) {
		return _bthreads_array[bind]
	}
/**
	Is the given bthread filtered
**/	
	function is_filtered(bth) {
		return !_type_filter[bth.type];
	}
	function bthread(node){
		this.node = node;
		this.id = this.node.getAttribute('id');
		_bthreads_num= _bthreads_num+1;
		this.bclass = this.node.getAttribute('class');
		if (this.bclass=="")
			this.bclass = "Dynamic BThread";
		this.index = _bthreads_num;
		_bthreads_array[this.index]=this;
		this.name = this.node.getAttribute('name');
		this.syncthreads = new Array();
		
		this.type = 0; //type of bthread - non active is default
		
		
		//for every syncpoint in every bthread, what is the next/prev active syncthread num;
		this.prevbsync=new Array(_bthreads_num+1);
		this.nextbsync=new Array(_bthreads_num+1);
		this.state=-1; //this substate it is on (or -1 if none)
		this.statemodified=false; //a boolean that tells if the state was modified in last sync
	}	
	
	function id_cmp(st1,st2) { //comparison between events, currently string based
		return (st1.name== st2.name)
	
	}
	
	/**
	Combining 2 bthreads, used for class grouping
	
	**/
	bthread.prototype.combine = function (bthr) {
		var it1=0;
		var it2 =0;
		this.type = Math.max(this.type, bthr.type);
		//fixing prev/next bsync
		for (var i=1;i<=_bthreads_num;i++) {
			this.prevbsync[i] = Math.max(this.prevbsync[i],bthr.prevbsync[i]);
			this.nextbsync[i] = Math.min(this.nextbsync[i],bthr.nextbsync[i]);
		}
		while(it1<this.syncthreads.length && it2 < bthr.syncthreads.length)
		{
			syncth1 = this.syncthreads[it1];
			syncth2 = bthr.syncthreads[it2];
			if (syncth1.syncindex> syncth2.syncindex) {
				_syncthreads_array[syncth2.syncindex][this.index] = syncth2;
				this.syncthreads.push(syncth2);
				it2++;
				}
			else if (syncth1.syncindex < syncth2.syncindex)
				it1++;
			else
				{
					//the syncth join itself
					syncth1.request.union(syncth2.request,id_cmp);
					syncth1.wait.union(syncth2.wait,id_cmp);
					syncth1.interrupt.union(syncth2.interrupt,id_cmp);
					syncth1.block.union(syncth2.block,id_cmp);
					syncth1.type = Math.max (syncth1.type,syncth2.type);
					syncth1.reinforcement += syncth2.reinforcement;
					if (syncth1.state!=-1 || syncth2.state!=-1)
						syncth1.state=-2; //complicated state
					_syncthreads_array[syncth1.syncindex][this.index]=syncth1;
					it1++;
					it2++;
				}
		}
		while (it2<bthr.syncthreads.length) { //take care of the rest
				syncth2 = bthr.syncthreads[it2];
				_syncthreads_array[syncth2.syncindex][this.index] = syncth2;
				this.syncthreads.push(syncth2);
				it2++;
		}
		
	}
	
	/**
		Parses xml and initializes all given bthreads
	**/
	function 	set_all_bthreads() {
		all = _doc.getElementsByTagName('hierarchyTbl');
				
		for (var j=0;j<all.length;j++) {
			nodes = all[j].getElementsByTagName('bthread')
			for (var k=0;k<nodes.length;k++) {
				var bth = set_bthread(nodes[k]);
			}
		}
	
	}
	

	function set_syncthread(sind,bth) {
		ind = bth.index;
		
		if (_syncthreads_array[sind][ind]!=undefined) {
			return _syncthreads_array[sind][ind];
			}
		else
			return new syncthread(sind,bth);
	}
	
	
	function get_syncthread(sind,bth) {
		ind = bth.index;
		if (_syncthreads_array[sind]==undefined) return _syncthreads_array[sind];
		return _syncthreads_array[sind][ind];
	}
	
		/**
			A cell, a bthread at a given syncpoint
		
		**/
	function syncthread(sind,bth) {
		ind = bth.index;
		this.bth = bth;
		this.syncindex = sind;
		//events
		this.request = new Array();
		this.wait = new Array();
		//Interrupt is needed for playgo.
		this.interrupt = new Array();
		this.block = new Array();
		//filters
		this.request_f = new Array();
		this.wait_f = new Array();
		//Interrupt is needed for playgo.
		this.interrupt_f = new Array();
		this.block_f = new Array();
		
		this.type=0; //0 for non-active, 1 for advanced, 2 for leader
		_syncthreads_array[sind][ind]=this;
		bth.syncthreads.push(this);
		this.reinforcement = 0;
	}
	
	/**
	returns the event list after grouping events if needed
	**/
	function get_grouped_event_list (array) {
		var array_typ1 = new Array();
		var array_other = new Array();
		//divide between events and filters
		for (var i=0;i<array.length;i++) {
			if (array[i].type==1) {
				array_typ1.push(array[i]);
				}
			else if (array[i].type>1)
				array_other.push(array[i]);
		}
		if (!_event_grouped || array_typ1.length<=1) {
			return array_other.concat(array_typ1); //removing 0 type filters
		}
		//find minimum event in 
		var min=4;
		var min_eve= array_typ1[0];
		for (var j=0;j<array_typ1.length;j++) {
			var eve = array_typ1[j];
			if (eve.get_status()<min) {
				min=eve.get_status();
				min_eve = eve;
			}	
		}
		var filter1 = new bfilter(min_eve.node,min_eve.sync);
		filter1.type=4;
		filter1.events = array_typ1;
		array_other.push(filter1);
		return array_other;
		
		
	}
	syncthread.prototype.get_request = function () {
		return get_grouped_event_list(this.request);
	}
	syncthread.prototype.get_wait = function () {
		return get_grouped_event_list(this.wait);
	}
	syncthread.prototype.get_interrupt = function () {
		return get_grouped_event_list(this.interrupt);
	}
		syncthread.prototype.get_block = function () {
		return get_grouped_event_list(this.block);
	}
	
	
	/**
		push execlusive
	**/
	Array.prototype.pushex = function ( needle ) {
	for (var i=0;i<this.length;i++) {
	//	if (this[i] == needle) return;
		if (id_cmp(this[i],needle)) return;
	}
	this.push(needle);
	}
	
	/**
		Exclusive union of two arrays, according to compare_funcs
	**/
	Array.prototype.union = function (array2, compare_func) {
	if (array2.length==0) return;
	for (var k=0;k<array2.length;k++) {
		var flag = false;
		for (var j=0;j<this.length;j++) {
			if (compare_func(this[j],array2[k])){
				flag = true;
				break;
			}
		}
		if (flag==false)
			this.push(array2[k]);
		}
	}
	/**
		initializes all syncthreads
	
	**/
	function 	set_all_syncthreads() {
		
		all_syncs = _doc.getElementsByTagName('syncpoint');
		for (i=1;i<=_syncpoints_num;i++) {
			_syncthreads_array[i] = new Array(_bthreads_num+1);
			_syncpoints_array[i] = new syncpoint(i);
			_syncpoints_array[i].event = new bevent(all_syncs[i-1].getElementsByTagName('event')[0]); //sets current event

			//see action selections (learning)
			if (all_syncs[i-1].getElementsByTagName('qvalues').length != 0) {
				qvalues_elm = all_syncs[i-1].getElementsByTagName('qvalues')[0].getElementsByTagName('event');
				for (j=0;j<qvalues_elm.length;j++) {
					evename = qvalues_elm[j].getAttribute('name');
					val = qvalues_elm[j].getAttribute('value');
					val = (Math.floor(val*100))/100; //2 digits accuracy
					_syncpoints_array[i].selectionActions.push(new actionselection(evename,val));
				}
			}
			
			
			
			requests = all_syncs[i-1].getElementsByTagName('dependencyTbl')[0].getElementsByTagName('request');
	
			var leader_flag = false; //leader was not found yet for the syncpoint
			for (j=1;j<=requests.length;j++) {
				request = requests[j-1];
							
				event_node = request.getElementsByTagName('event')[0];
				var event = new bevent(event_node,_syncpoints_array[i]);
				bth = get_bthread_by_id(request.getElementsByTagName('bthread')[0].getAttribute('id'));

				event.request.push(bth);
				st = set_syncthread(i,bth);
				st.request.pushex(event);
				all_blocked = request.getElementsByTagName('block');
				if (_syncpoints_array[i].event.id == event.id) {
					 if (!leader_flag)  {
						leader_flag = true;
						st.type=2;
					} else  {
						st.type=1;
					}
				}
				for (var k=0;k<all_blocked.length;k++) {
					b_bth = get_bthread_by_id(all_blocked[k].getElementsByTagName('bthread')[0].getAttribute('id'));
					b_filter = get_filter(all_blocked[k].getElementsByTagName('filter')[0],_syncpoints_array[i]);
					b_filter.events.pushex(event);
					st2 = set_syncthread(i,b_bth);
					st2.block.pushex(b_filter);
					event.block.push(b_filter);
				}
				all_interrupted = request.getElementsByTagName('interrupt');
				for (var k=0;k<all_interrupted.length;k++) {
					w_bth = get_bthread_by_id(all_interrupted[k].getElementsByTagName('bthread')[0].getAttribute('id'));
					w_filter = get_filter(all_interrupted[k].getElementsByTagName('filter')[0],_syncpoints_array[i]);
					w_filter.events.pushex(event);
					st2 = set_syncthread(i,w_bth);
					st2.interrupt.pushex(w_filter);
					if (_syncpoints_array[i].event.id == event.id && st2.type!=2)  {
						st2.type=1;
					}
					event.interrupt.push(w_filter);
				}
				all_waited = request.getElementsByTagName('watch');
				for (var k=0;k<all_waited.length;k++) {
					w_bth = get_bthread_by_id(all_waited[k].getElementsByTagName('bthread')[0].getAttribute('id'));
					w_filter = get_filter(all_waited[k].getElementsByTagName('filter')[0],_syncpoints_array[i]);
					w_filter.events.pushex(event);
					st2 = set_syncthread(i,w_bth);
					st2.wait.pushex(w_filter);
					if (_syncpoints_array[i].event.id == event.id && st2.type!=2)  {
						st2.type=1;
					}
					event.wait.push(w_filter);
				}
			}
			// set wait/interrupted/block which are not requested
			bthreads = all_syncs[i-1].getElementsByTagName('hierarchyTbl')[0].getElementsByTagName('bthread');
			for (j=1;j<=bthreads.length;j++) {
				bthread_node = bthreads[j-1];
				bth = get_bthread_by_id(bthread_node.getAttribute('id'));
				st2 = set_syncthread(i,bth);
				all_blocked = bthread_node.getElementsByTagName('blocked');
				for (var k=0;k<all_blocked.length;k++) {
					b_filter = get_filter(all_blocked[k],_syncpoints_array[i]);
					subs = all_blocked[k].getElementsByTagName('sub-filter');
					for (var t=0;t<subs.length;t++) {
							var eve = new bevent(subs[t],_syncpoints_array[i]);
							b_filter.events.pushex(eve);
					}
					st2.block.pushex(b_filter);
				}
				all_interrupted = bthread_node.getElementsByTagName('interrupted');
				for (var k=0;k<all_interrupted.length;k++) {
					b_filter = get_filter(all_interrupted[k],_syncpoints_array[i]);
					subs = all_interrupted[k].getElementsByTagName('sub-filter');
					for (var t=0;t<subs.length;t++) {
							var eve = new bevent(subs[t],_syncpoints_array[i]);
							b_filter.events.pushex(eve);
					}
					st2.interrupt.pushex(b_filter);
				}
				all_watched = bthread_node.getElementsByTagName('watched');
				for (var k=0;k<all_watched.length;k++) {
					b_filter = get_filter(all_watched[k],_syncpoints_array[i]);
					subs = all_watched[k].getElementsByTagName('sub-filter');
					for (var t=0;t<subs.length;t++) {
							var eve = new bevent(subs[t],_syncpoints_array[i]);
							b_filter.events.pushex(eve);
					}
					st2.wait.pushex(b_filter);
				}
				
			}
						//set current state, if it exists (learning)
			if (all_syncs[i-1].getElementsByTagName('state').length != 0) {
				substates_elm = all_syncs[i-1].getElementsByTagName('state')[0].getElementsByTagName('substate');
				for (j=0;j<substates_elm.length;j++) {
					bthname = substates_elm[j].getAttribute('bthread');
					val = substates_elm[j].getAttribute('value');
					_syncpoints_array[i].state.push(new substate(bthname,parseFloat(val)));
					bthr = get_bthread_by_name(bthname);
					var syncth = get_syncthread(i,bthr);
					if (syncth==undefined) break;
					syncth.state = val;
					last_syncth = get_syncthread(i-1,bthr);
					if (last_syncth==undefined)
						syncth.statemodified = true;
					else {
						if (syncth.state != last_syncth.state)
							syncth.statemodified = true;
						
					}
				}
			}
			
		}
	}
	
	function get_syncpoint(index) {
		return _syncpoints_array[index];
	
	}
	function syncpoint(index) {
		this.index = index;
		this.event = 0;
		this.filterArray = new Array();
		this.state = new Array(); //an array of substates (for learning)
		this.selectionActions = new Array(); //array of selection events (for learning);
	}
	

	/**
		return true if the state was modified form last state
	**/
	syncpoint.prototype.is_state_modified = function () {
		if (this.index==1) return true;
		state2 = _syncpoints_array[this.index-1].state;
		if (this.state.length!= state2.length) return true;
		for (var i=0;i<this.state.length;i++)
			if (this.state[i].bthname != state2[i].bthname || this.state[i].status != state2[i].status)
			{
				return true;
				}
		return false;
	}
	
	/**
		returns a string representation of the sync state
	**/
	syncpoint.prototype.display_state = function () {
		var str = "";
		for (var i=0;i<this.state.length;i++) {
			str= str+ "\n"+this.state[i].bthname+ " : "+ this.state[i].status;
		}
		return str;
	
	}
	
	
	/**
	both class are for learning
	**/
	function substate(bthname,status) {
		this.bthname = bthname;
		this.status = status; //status int number
	}	
	
	function actionselection(evename,value) {
		this.evename = evename;
		this.value = value;
	}
	
	
	function get_filter(node,sync) {
		id = node.getAttribute('id');
		for (var j=0;j<sync.filterArray.length;j++)
			if (sync.filterArray[j].id == id)
				return sync.filterArray[j];
		new_filt =  new bfilter(node,sync);
		sync.filterArray.push(new_filt);
		return new_filt;
	
	}
	
		
	function bevent(node,sync) {
		this.node = node;
		this.id = node.getAttribute('id');

		this.name = node.getAttribute('name');


		this.sync = sync;
		this.request = new Array(); //who requested it
		this.wait = new Array(); //who waits for it
		//Interrupt is needed for playgo.
		this.interrupt = new Array(); //who interrupt for it
		this.block = new Array(); //who blocks it
		this.events = new Array();
		this.type=1;
		this.status=-1; //blocked, etc.
	}
	
	/**
		filter class
	**/
	function bfilter(node,sync) {
		this.id = node.getAttribute('id');
		this.name = node.getAttribute('name');
		this.sync = sync;
		this.node = node;
		this.events = new Array();
		typ = node.getAttribute('type')
		if (typ=='Event')
			this.type=1;
		else if (typ=='Empty')
			this.type=0;
		else if (typ=='AbstractFilter')
			this.type=2;
		else this.type=3;
		this.status = -1;
	}
	/**
		returns the status of the event, 1 = leader, 2 = blocked 4 = otherwise
	**/
	bevent.prototype.get_status = function () {
		if (this.status!=-1) return this.status;
		if (id_cmp(this,this.sync.event)) { this.status=1; return 1;}
		if (this.block.length!=0) {this.status= 2; return 2;}
		for (var i=0;i<this.sync.selectionActions.length;i++) {
			if (this.sync.selectionActions[i].evename == this.name)
				{ this.status= 3; return 3;} //this was a selection event
		}
		this.status =4;
		return 4;
	}
	
	
	bfilter.prototype.get_status = function () {
		if (this.status!=-1) return this.status;
		if (this.type==1)
			{
				if (id_cmp(this,this.sync.event)) return 1;
				this.status = 4;
				return 4;
			}
		var min=4;
		for (var j=0;j<this.events.length;j++) {
			eve = this.events[j];
			if (eve.get_status()<min) {
				min=eve.get_status();
				}
		}
		this.status = min;
		return this.status;
	}
	
	
	
	
	
	
	
	
/**
	Checks the types of the bthreads (non-active, etc.)
**/	
function set_bthread_types() {

	for (var i=1;i<=get_table_width();i++)
	{
		var bthr = get_bthread(i);
			bthr.type=2; //default is active
		var non_active_flag=1;
		var preactive_flag = 1;
		var cur=0;
		var prev_sync_num=0;
		for (var j=0;j<bthr.syncthreads.length;j++)
		{
			var syncth = bthr.syncthreads[j];
			
			if (syncth.type>=1) {
			//setting prev and next syncth
			for (var k=prev_sync_num+1;k<=syncth.syncindex;k++)
				bthr.nextbsync[k]=syncth.syncindex;
			for (var k=prev_sync_num;k<syncth.syncindex;k++)
				bthr.prevbsync[k]=prev_sync_num;
			prev_sync_num = syncth.syncindex;
			}
				
			if (syncth.type==2) 
			{
				bthr.type=3; //leader bthread
				non_active_flag=0;
				preactive_flag=0;
			}
			if (syncth.type==1)
				non_active_flag=0;
			for (k=0;k<syncth.block.length;k++)  //check for dominant blockers
				if (syncth.block[k].events.length!=0) {
					non_active_flag=0;
					preactive_flag=0;
					}
			if (syncth.request.length!=0) {
				preactive_flag = 0;
				non_active_flag=0;
			}
		
		}
		//sets the prev of whats left
		for (var k=prev_sync_num;k<=get_total_syncpoints_num();k++) {
			bthr.prevbsync[k]=prev_sync_num;
			bthr.nextbsync[k]= get_total_syncpoints_num()+1;
		}
				
		if (non_active_flag==1)
			bthr.type=0; //non-active
		else if (preactive_flag==1)
			bthr.type=1;
	}
	//count how many of each type
	for (var i=0;i<4;i++)
		_bthread_type_counters[i] = 0;
	for (var i=1;i<=get_table_width();i++) {
		var bthr = get_bthread(i);
		_bthread_type_counters[bthr.type]++;
	}

	
}
/**
	returns the number of each type of bthread for statistics
**/
function get_num_bthread_type() {
	return _bthread_type_counters;
}
/**
	returns the total number of filtered bthreads
**/

function get_num_bthread_filter() {
	all = get_num_bthread_type();
	counter =0;
	for (var i=0;i<all.length;i++)
		if(_type_filter[i])
			counter+=all[i];
	return counter;
}

/**
	Returns the leader in the given row
**/
function get_leader_in_row(row) {
		var list = get_filtered_bthread_list ();
		var sync = get_syncpoint(row);
		var count=0;
		for (i=0;i<list.length;i++) {
			var bthr = list[i];
			syncth = get_syncthread(row,bthr);
			if (syncth == undefined)
				continue;
			if (syncth.type == 2)
				return i+1;
		}
		return 0;

}
function filter_bthread_type(array) {
	_type_filter = array;
}

/**
	Sets the reinforcements data
**/
function set_reinforcements() {
		all_reinforce = _doc.getElementsByTagName('reinforcements');
		for (var i=0;i<all_reinforce.length;i++) {
			var reinforce = all_reinforce[i];
			var sync_num = reinforce.getAttribute('number');
			all_threads = reinforce.getElementsByTagName('bthread');
			for (var j=0;j<all_threads.length;j++) {
				cur_thread = all_threads[j];
				cur_thread_id = cur_thread.getAttribute('id');
				bth = get_bthread_by_id(cur_thread_id);
				if (bth==0)
					bth= get_bthread_by_name(cur_thread_id); //enables putting either name or java id
				val = cur_thread.getAttribute('value');
				syncth = get_syncthread(sync_num,bth);
				syncth.reinforcement = parseFloat(val);
			}
		}
}


/**
	Loads the xml, needs to be called once for every file.
**/
function load_xml(name) {
	var xmlDoc;
	// code for IE
	if (window.ActiveXObject)
	{
		xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
		xmlDoc.async=false;
		xmlDoc.load(name);
	}
	// code for Mozilla, Firefox, Opera, etc.
	else if (document.implementation.createDocument)
	{
		var xmlhttp = new window.XMLHttpRequest();
		xmlhttp.open("GET",name,false);
		xmlhttp.send(null);
		xmlDoc = xmlhttp.responseXML.documentElement;
	
//		xmlDoc=document.implementation.createDocument("","",null);
	}
	else
	{
		alert('Your browser cannot handle this script');
	}
	_doc=xmlDoc;
	
	_eventtrace_style = _doc.getAttribute('style');
	// For backward compatibility.
	if (_eventtrace_style == null) {
		_eventtrace_style = "BPJ";
	}
	
	all = _doc.getElementsByTagName('syncpoint');
	_syncpoints_num = all.length;
	_bthreads_array = new Array();
	_syncthreads_array= new Array(_syncpoints_num+1);
	_syncpoints_array = new Array(_syncpoints_num+1);
	_bthreads_num=0;
	set_all_bthreads();
	set_all_syncthreads();
	set_bthread_types();
	set_reinforcements();
	_startpoint=1;
	_endpoint = _syncpoints_num;

}


/**
	returns the current start syncpoint
**/
function get_start_point() {
	return _startpoint;
}
/**
	returns the current end syncpoint
**/
function get_end_point() {
	return _endpoint;
}

/**
	returns the number of bthread classes
**/
function get_classes_num() {
	var filtered_array = new Array();
	
	for (i=1;i<_bthreads_array.length;i++)
	{
		var bthr = get_bthread(i);
		var flag=0;
		for (var j=0;j<filtered_array.length; j++)
		{
			if (filtered_array[j].bclass == bthr.bclass)
			{
					flag=1
			}
		}
		if (flag==0)
			filtered_array.push(bthr);
	}
	return filtered_array.length;
}
function get_total_syncpoints_num() {
	return _syncpoints_num;
}
/**
sets the run to start from the given start

**/
function constrain_run(start) {
	end = start + _sync_in_page-1;
	if (end < _syncpoints_num) _endpoint = end;
	else _endpoint=_syncpoints_num;
	if (start < _syncpoints_num && start<=end) _startpoint = start;
	else _startpoint=_endpoint-_sync_in_page+1;
}

function get_table_width() {
	return _bthreads_num;
}
function get_table_height() {
	return _endpoint-_startpoint+1;
}
