/********************************
bpj_visual.js
The visual model file of the visualization
Used for Showing Svg graphics and drawing the tables and data on the screen
Written by Nir Eitan, Wiezmann Is., 2010
***********************************/
var _width = 150;
var _height = 90;
var _xmar = 20;
var _ymar = 30;
var _xspace = _xmar+_width;
var _yspace = _ymar+_height;
var _paper;
var _nx;
var _ny;
var _rects_array;
var _collapse_buttons;
var _sizes;
var _style = 0;
var _chosen_row=0; //for shift-using
var _first_width=210; //the first column width
var _static_view= false;

/**
	Visualization style, can be 0 or 1
**/
function set_style(num) {
	_style=num;
}
/**
	Sets Static view mode (not showing dynamic buttons), true or false.
**/
function set_static_view(bool) {
	_static_view=bool;
}

/**
Checks box sizes, for canvas initialization
**/
function update_sizes(sync,bthr) {
	yind = sync.index;
	syncth = get_syncthread(yind,bthr);
	if (syncth == undefined) {
		if (_sizes[yind]==undefined) 
			_sizes[yind]=0;
	return;
	}
	
	requested = syncth.get_request();
	if (requested.length==0) {requested = new Array(1);}
	
	waited = syncth.get_wait(); 
	if (waited.length==0) {waited = new Array(1);}

	interrupted = syncth.get_interrupt(); 
	if (interrupted.length==0) {interrupted = new Array(1);}
	
	blocked = syncth.get_block(); 
	if (blocked.length==0) {blocked = new Array(1);}

	total = requested.length+waited.length+blocked.length;

	// If and when interrupted in BPJ, remove this IF.
	if (_eventtrace_style == "playgo") {
		total += interrupted.length;
	}
	
	if (_sizes[yind]==undefined || _sizes[yind]<total)
		_sizes[yind]=total;
}


/**
	Draws a cell
**/
function draw_bpj_rect(sync,bthr,xind) {
	var all_rect = _paper.set();
	yind = sync.index;
	syncth = get_syncthread(yind,bthr);
	extra = 0; //extra yspace
	for (var k=get_start_point();k<yind;k++)
		extra= extra+_sizes[k]-3;
	extra= extra*_height/3;	
	if (syncth == undefined) return all_rect; //no rect
	
	var requested = syncth.get_request();
	if (requested.length==0) {requested = new Array(1);}

	var waited = syncth.get_wait(); 
	if (waited.length==0) {waited = new Array(1);}

	var interrupted = syncth.get_interrupt(); 
	if (interrupted.length==0) {interrupted = new Array(1);}

	var blocked = syncth.get_block(); 
	if (blocked.length==0) {blocked = new Array(1);}

	var total = requested.length+waited.length+blocked.length;

	// If and when interrupted in BPJ, remove this IF.
	if (_eventtrace_style == "playgo") {
		total += interrupted.length;
	}

	all_rect.total = total;
	text_x = _xspace*(xind-0.5)-_xmar/2+10+_first_width;
	all_rect.box = _paper.set();
		//outer line
	all_rect.line = 0;

	if (get_syncthread(yind+1,bthr)!=undefined)
	{
		x2=(xind-1)*_xspace+_first_width+_width/2;
		y1= _yspace*sync.index+total*_height/3+extra;
		y2= y1+_ymar+(_sizes[yind]-total)*_height/3;
		path = _paper.path("M"+x2+" "+y1+"L"+x2+" "+y2);
		all_rect.push(path);
		if (syncth.type==2)
			path.attr("stroke-width",6);
		else if (syncth.type==1)
			path.attr("stroke-width",3);
		else
		{
			path.attr("stroke-width",3);
			path.attr("stroke-dasharray",".");
		}
		all_rect.line=path;
	}
	//outer rect
	rect= _paper.rect(_xspace*(xind-1)+_first_width,_yspace*yind+extra,_width,_height*total/3,3);
	all_rect.push(rect);
	all_rect.box.push(rect);
	if (_style==0) {
	if (syncth.type==2)
		rect.attr("fill", "#6495ED");
	else if (syncth.type==1)
		rect.attr("fill", "#ddf");
	else
		rect.attr("fill", "#eee");
	} else if (_style>=1) {
			if (syncth.type==2)
			rect.attr("fill", "#9ac0e7");
		else if (syncth.type==1)
			rect.attr("fill", "#e1ecf8");
		else
			rect.attr("fill", "#fafafa");
	
	}
	rect.attr("stroke-opacity",0);	
	
	
		//Creating the midlines
	x1 = _xspace*(xind-1)+_first_width;
	x2= _xspace*(xind-1)+_first_width+_width;
	
	for (var j=1;j<total;j++) {
		y=_yspace*yind+j*_height/3+extra;
		path = _paper.path("M"+x1+" "+y+"L"+x2+" "+y); 
		all_rect.push(path);
		path.attr("stroke-width",0.5);
	}

	

	
	//creating the events sets display
	var extra_array = _paper.set();
	var popup_array = _paper.set();
	cur_y = _yspace*yind+extra;
	for (var j=0;j<requested.length;j++) 
	{
		var event = requested[j];

		if (_style!=2) {
			small_rect = _paper.rect(x1,cur_y,_width/6,_height/3,0);			
			}
		else  {
			small_rect = _paper.rect(x1,cur_y,_width/36,_height/3,0);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);	
		}
		if (_style!=1 && _style!=4) //color
			small_rect.attr("fill", "#0f0");
		if (_style==3) {
			small_rect.attr("opacity",0.15);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);
		}
		text =_paper.text(x1+_width/12,cur_y+_height/6,"R");
		if (_style==4) {
							small_rect.attr("stroke","#fff");
			small_rect.attr("fill","#000");
				text.attr("fill","#00CD00");
		
		}

		
		all_rect.push(small_rect);
		
		all_rect.push(text);
		text.attr("font-size",7+Math.floor(_width/24));
		arrayevent =write_event_array(event,sync,text_x,cur_y+_height/6);
		extra_array.push(arrayevent.elem);
		popup_array.push(arrayevent.popup);
		all_rect.push(arrayevent);
		cur_y = cur_y+_height/3;
	}
	

	for (var j=0;j<waited.length;j++) {
		var event = waited[j];
		if (_style!=2) {
			small_rect = _paper.rect(x1,cur_y,_width/6,_height/3,0);			
			}
		else  {
			small_rect = _paper.rect(x1,cur_y,_width/36,_height/3,0);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);	
		}
		if (_style!=1 && _style!=4) //color
			small_rect.attr("fill", "#ff0");
		if (_style==3) {
			small_rect.attr("opacity",0.15);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);
		}
		text =_paper.text(x1+_width/12,cur_y+_height/6,"W");
		if (_style==4) {
					small_rect.attr("fill","#000");
					small_rect.attr("stroke","#fff");
				text.attr("fill","#FFB90F");
		
		}
		
		text.attr("font-size",7+Math.floor(_width/24));
		all_rect.push(small_rect);
		all_rect.push(text);
		arrayevent= write_event_array(event,sync,text_x,cur_y+_height/6);
		extra_array.push(arrayevent.elem);
		popup_array.push(arrayevent.popup);
		all_rect.push(arrayevent);
		cur_y = cur_y+_height/3;
	}
	
	// If and when interrupted in BPJ, remove this IF.
	if (_eventtrace_style == "playgo") {
		for (var j=0;j<interrupted.length;j++) {
			var event = interrupted[j];
			if (_style!=2) {
				small_rect = _paper.rect(x1,cur_y,_width/6,_height/3,0);			
			}
			else  {
				small_rect = _paper.rect(x1,cur_y,_width/36,_height/3,0);
				small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
				all_rect.push(small_rect2);	
			}
			if (_style!=1 && _style!=4) //color
				small_rect.attr("fill", "#ff0");
			if (_style==3) {
				small_rect.attr("opacity",0.15);
				small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
				all_rect.push(small_rect2);
			}
			text =_paper.text(x1+_width/12,cur_y+_height/6,"I");
			if (_style==4) {
						small_rect.attr("fill","#ff0");
						small_rect.attr("stroke","#0ff");
					text.attr("fill","#abcdef");
			}
			
			text.attr("font-size",7+Math.floor(_width/24));
			all_rect.push(small_rect);
			all_rect.push(text);
			arrayevent= write_event_array(event,sync,text_x,cur_y+_height/6);
			extra_array.push(arrayevent.elem);
			popup_array.push(arrayevent.popup);
			all_rect.push(arrayevent);
			cur_y = cur_y+_height/3;
		}
	}
	
	for (var j=0;j<blocked.length;j++) {
		var event = blocked[j];
		if (_style!=2) {
			small_rect = _paper.rect(x1,cur_y,_width/6,_height/3,0);			
			}
		else  {
			small_rect = _paper.rect(x1,cur_y,_width/36,_height/3,0);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);	
		}
		if (_style!=1 && _style!=4) //color
			small_rect.attr("fill", "#f00");
		if (_style==3) {
			small_rect.attr("opacity",0.15);
			small_rect2 = _paper.rect(x1,cur_y,_width/6,_height/3,0); //boundary rect
			all_rect.push(small_rect2);
		}
		text =_paper.text(x1+_width/12,cur_y+_height/6,"B");
				if (_style==4) {
									small_rect.attr("stroke","#fff");
							small_rect.attr("fill","#000");
				text.attr("fill","#CD0000");
		
		}
		
		text.attr("font-size",7+Math.floor(_width/24));
		all_rect.push(small_rect);
		all_rect.push(text);
		arrayevent = write_event_array(event,sync,text_x,cur_y+_height/6);
		extra_array.push(arrayevent.elem);
		popup_array.push(arrayevent.popup);
		all_rect.push(arrayevent);
		cur_y = cur_y+_height/3;
		
	}
	//outer rect - reborder
	rect= _paper.rect(x1,_yspace*yind+extra,_width,_height*total/3,3);
	all_rect.push(rect);
	if (syncth.type==2) {
		rect.attr("stroke-width",5);
		

		/* if (_style==0) {
			rect.attr("stroke-width",5); //bold
			//rect.attr("stroke-width",2);
			star = _paper.path("M 46.296296,51.906272 L 31.916351,42.474649 L 17.502712,51.8547 L 22.029072,35.264028 L 8.654054,24.454438 L 25.831443,23.632463 L 31.978866,7.5717174 L 38.068716,23.65438 L 55.243051,24.537884 L 41.829396,35.299492 L 46.296296,51.906272 z");
			star.attr("fill", "#2f0");
			star.translate(x1-30,_yspace*yind+extra-30);
			star.scale(0.45,0.45);
			all_rect.box.push(star);
			all_rect.push(star);
		} */
			
	}
	else	if (syncth.type==0)
		{
			rect.attr("stroke-width",2);
			rect.attr("stroke-dasharray","--");
			rect.attr("stroke-opacity",0.2);
		}
	else {
		rect.attr("stroke-width",2);
	}
	all_rect.box.push(rect);
	all_rect.elem = extra_array;
	all_rect.popup = popup_array;
	//all_rect.push(popup_array);
	
	all_rect.learningre = _paper.set(); //a set of the learning elements in this rect
	all_rect.learningstate = _paper.set(); 
		//creating the state
	if (syncth.state!=-1 && syncth.statemodified==true) {
		rect_y = cur_y+_ymar/4-_yspace;
		rect_x = x1+1*_width/6-20;
		small_rect = _paper.rect(rect_x,rect_y,_width/3,_ymar/2,2);
		small_rect.attr("fill","#aaa");	
		small_rect.attr("opacity",0.9);
		ipath = "M15.067,2.25c-5.979,0-11.035,3.91-12.778,9.309h3.213c1.602-3.705,5.271-6.301,9.565-6.309c5.764,0.01,10.428,4.674,10.437,10.437c-0.009,5.764-4.673,10.428-10.437,10.438c-4.294-0.007-7.964-2.605-9.566-6.311H2.289c1.744,5.399,6.799,9.31,12.779,9.312c7.419-0.002,13.437-6.016,13.438-13.438C28.504,8.265,22.486,2.252,15.067,2.25zM10.918,19.813l7.15-4.126l-7.15-4.129v2.297H-0.057v3.661h10.975V19.813z";
		icon = _paper.path(ipath);
		icon.translate(rect_x-4,rect_y-10);
		icon.scale(0.4,0.4);
		icon.attr("fill","#226");
		if (syncth.state!=-2) { //merged complicated state
			text = _paper.text(rect_x+30,rect_y+5,syncth.state);
			all_rect.push(text);
			all_rect.learningstate.push(text);
		}
		all_rect.push(icon);
		all_rect.push(small_rect);
		all_rect.learningstate.push(icon);
		all_rect.learningstate.push(small_rect);
		
		
	}
	
	//now the reinforcements are visualized (in case they exist)
	re = syncth.reinforcement;
	re = (Math.floor(re*100))/100.0;
	if (re!=0) {
		rect_y = cur_y+_ymar/4;
		rect_x = x1+2*_width/3-10;
		back_rect =  _paper.rect(rect_x,rect_y,_width/3,_ymar/2,2);
		back_rect.attr("fill","#fff");
		small_rect = _paper.rect(rect_x,rect_y,_width/3,_ymar/2,2);
		//small_rect.attr("fill","#aaa");	
		small_rect.attr("opacity",0.4);

		var path;
		var color;
		if (re>0) {
			mpath = "M25.979,12.896 19.312,12.896 19.312,6.229 12.647,6.229 12.647,12.896 5.979,12.896 5.979,19.562 12.647,19.562 12.647,26.229 19.312,26.229 19.312,19.562 25.979,19.562z";
			color = "#9932CC";
			}
		else {
			mpath = "M25.979,12.896,19.312,12.896,5.979,12.896,5.979,19.562,25.979,19.562z"
			color = "#FF8C00";
			re = -1*re; //making it positive (as the icon shows its negative)
			}
		small_rect.attr("fill",color);	
		icon = _paper.path(mpath);
		icon.attr("fill",color);
		icon.translate(rect_x-4,rect_y-9);
		icon.scale(0.5,0.5);
		
		text = _paper.text(rect_x+30,rect_y+6,re);
		all_rect.push(text);
		all_rect.push(icon);
		all_rect.push(small_rect);
		all_rect.push(back_rect);
		all_rect.learningre.push(text);
		all_rect.learningre.push(small_rect);
		all_rect.learningre.push(icon);
		all_rect.learningre.push(back_rect);
	}
	
	all_rect.translate(0,-(get_start_point()-1)*_yspace);
	return all_rect;
}

/**
Shows an event in <xpos,ypos>
**/
function write_event(event,sync,xpos,ypos) {
	var type = event.get_status();
	name = event.name;
	max_text = Math.floor(_width/13);
	tooltip = 0;
	var twoelems = _paper.set();
	twoelems.popup = _paper.set();
	if (name.length>max_text)
	{
		name = name.substr(0,max_text-1);
		name = name.concat("...");
		tooltip=1;
		
	}
	text =_paper.text(xpos+2,ypos,name);
		text.attr("font-size",12);

	twoelems.push(text);
	if (type==1) { //active
		text.attr("font-weight",'bolder');
		star = _paper.path("M 46.296296,51.906272 L 31.916351,42.474649 L 17.502712,51.8547 L 22.029072,35.264028 L 8.654054,24.454438 L 25.831443,23.632463 L 31.978866,7.5717174 L 38.068716,23.65438 L 55.243051,24.537884 L 41.829396,35.299492 L 46.296296,51.906272 z");
		star.attr("fill", "#2f0");
		star.translate(xpos-_width/3-29,ypos-30);
		star.scale(0.25,0.25);
		twoelems.push(star);
		path.attr("fill", "#ff0");
		if (sync.selectionActions.length>1) {  //if there was a selection choice
			backicon = _paper.rect(xpos-_width/3-3,ypos-7,20,20); //placeholder for having a larger click zone
			backicon.attr("fill", "#ff0");
			backicon.attr("opacity",0);
			backicon.attr("stroke-width",0);
			twoelems.push(backicon);
			popup = register_decisions_popup (xpos+60,ypos-20,backicon,sync,event);
			twoelems.popup.push(popup);
			twoelems.push(popup);
		}
	}
	// TODO: probably nice to add a nice icon for cold-blocked...
	else if (type==2) { //blocked
		block_size = 8;
		block_rect= _paper.rect(xpos-_width/3,ypos-block_size/2,block_size,block_size,1);
		twoelems.push(block_rect);
		block_rect.attr("fill", "#f00");
	}
	
	else if (type==3) { //selection
		ipath = "M26.268,22.562l3.514-2.03l-3.514-2.028l-3.515-2.028v2.256c-0.922-0.002-2.45-0.002-2.883-0.002c-1.28-0.03-2.12-0.431-2.994-1.148c-1.303-1.07-2.415-2.997-3.756-4.853c-0.682-0.923-1.442-1.839-2.454-2.575C9.664,9.415,8.355,8.902,6.9,8.912H0.593v3.604H6.9c0.877,0.016,1.432,0.302,2.189,1.012c1.12,1.055,2.212,3.072,3.718,4.983c1.476,1.893,3.747,3.804,7.041,3.823h2.905v2.256L26.268,22.562zM11.401,8.912c0.04,0.028,0.082,0.053,0.121,0.082c1.202,0.874,2.07,1.947,2.757,2.88c0.158,0.218,0.298,0.428,0.448,0.642h8.026v2.257l3.515-2.028l3.514-2.029l-3.514-2.029l-3.515-2.03v2.255H11.401z"
		icon = _paper.path(ipath);
		icon.translate(xpos-_width/3-10,ypos-15);
		twoelems.push(icon);
		icon.scale(0.5,0.5);
		icon.attr("fill", "#00f");
		backicon = _paper.rect(xpos-_width/3-3,ypos-7,20,20);
		backicon.attr("opacity",0);
		backicon.attr("fill", "#ff0");		
		backicon.attr("stroke-width",0);
		twoelems.push(backicon);
		popup = register_decisions_popup (xpos+60,ypos-20,backicon,sync,event);
		twoelems.popup.push(popup);
		twoelems.push(popup);
	}
	else {
	}
	//tooltip
	if (tooltip==1) {
		popup = register_popup(xpos,ypos,text,event.name,1); //extra items (not auto-shown)
		twoelems.popup.push(popup);
		twoelems.push(popup);
	}
	return twoelems
}
/**
	Register a popup screen
**/
function register_popup(xpos,ypos,button, text, numoflines) {
	button.tooltip = _paper.set();
	rect= _paper.rect(xpos-_width/2,ypos-2*_height/3,_width*2,(numoflines+1)*_height/6);
	rect.attr("fill","#DBDB70");
	rect.hide();
	rect.attr("opacity",0.9);
	intext = _paper.text(xpos+_width/2,ypos-_height/2+(numoflines)*_height/12-5,text);
	intext.attr("font-size",6+Math.floor(_width/30));
	intext.hide();
	button.tooltip.push(rect);
	button.tooltip.push(intext);
	button.mouseover (function() {
		this.tooltip.show();	
		this.tooltip.toFront();
	});
	button.mouseout (function() {
		this.tooltip.hide();
		this.tooltip.disabled =1;
	
	});
	return button.tooltip;
}

/** 
show the learning decision and q-values. xpos,ypos - locations of tooltiptext. button = the button that activates the popup. sync = the current syncpoint, event = the current event
**/
function register_decisions_popup (xpos,ypos,button,sync,event) {
	button.tooltip = _paper.set();
	numoflines = sync.selectionActions.length;
	var rect= _paper.rect(xpos-_width/2,ypos-2*_height/3,_width*2,(numoflines+1)*_height/6);
	rect.attr("fill","#DBDB7b");
	rect.hide();
	rect.attr("opacity",0.9);
		button.tooltip.push(rect);
	for (var i=0;i<sync.selectionActions.length;i++) {
			if (event.name == sync.selectionActions[i].evename) //the current event is shown	
			{
				color_rect =  _paper.rect(xpos-_width/2,ypos-_height/2+i*15-8,_width*2,15);
				color_rect.attr("fill","#bff");
				color_rect.attr("opacity",0.9);
				color_rect.hide();
				button.tooltip.push(color_rect);
			
			}
			intext = _paper.text(xpos+_width/2,ypos-_height/2+i*15,sync.selectionActions[i].evename + " : "+ sync.selectionActions[i].value);
			intext.attr("font-size",6+Math.floor(_width/30));
			intext.attr("opacity",1);
			if (sync.selectionActions[i].evename==sync.event.name) 
				intext.attr("font-weight",'bolder'); //bold for active event
			intext.hide();
			button.tooltip.push(intext);
			

			
	}

	button.mouseover (function() {
		this.tooltip.show();	
		this.tooltip.toFront();
	});
	button.mouseout (function() {
		this.tooltip.hide();
		this.tooltip.disabled =1;
	
	});
	return button.tooltip;
	

}
/**
	Show a filter
**/
function write_event_array(event,sync,xpos,ypos) {
	if (event==undefined || event.type==0) return 0;
	all_rect = _paper.set();
	twoelems = write_event(event,sync,xpos,ypos);
	all_rect.push(twoelems);
	all_rect.popup = _paper.set();
	if (twoelems.popup!=undefined) 
		all_rect.popup = twoelems.popup;
	else
		all_rect.elem = 0; //extra elements to hide
	//button
	
	//do something about the rest of the array
	if (event.events.length > 0 && event.type>=2)
	{
	
		rest = event.events.length;
		//new_x = xpos+_xspace/2-_xmar-_width*1/10
		new_x = xpos+_width/2-_width*1/10-10;
		var butt = _paper.rect(new_x,ypos-_height/6,_width/10,_height/3);
		all_rect.push(butt);
		butt.attr("fill", "#000");
		butt.attr("fill-opacity",0.3);
		arrow1 = _paper.path("M-6 -6L0 0L-6 6");
		arrow1.scale(_width/180,1);
		arrow1.translate(new_x+_width/16+3,ypos+4);
		arrow1.attr("stroke","#fff");
		arrow2 = _paper.path("M-6 -6L0 0L-6 6");
		arrow2.scale(_width/180,1);
		arrow2.translate(new_x+_width/20+3,ypos+4);
		arrow2.attr("stroke","#fff");
		all_rect.push(arrow1);
		all_rect.push(arrow2);
		textn = _paper.text(new_x+_width/20,ypos-_height/12,rest);
		textn.attr("fill","#fff");
		all_rect.push(textn);
		
		butt.activated = 0;
		
		
		butt.set = _paper.set();
		
		//buttons elements
		butt.elem = _paper.set();
		butt.popup = _paper.set();
		bx = new_x+_xspace/4;
		by = ypos-_yspace/6;
		rect= _paper.rect(bx,by,_width*3/4,_height*(rest)/3);
		all_rect.push(rect);
		
		extra_rect= _paper.set();
		butt.elem.push(rect);
		rect.attr("fill","#999");
		rect.attr("fill-opacity",0.9);
		//rect.attr("stroke-width",2);
		if (event.type==2) //Abstract
			rect.attr("stroke-dasharray","-");
		line = _paper.path("M0 0L-30 0");
		line.translate(bx,ypos);
		line.attr("stroke-width",5);
		all_rect.push(line);

		butt.elem.push(line);
		by = by+_height/6;
		bx = bx + _width*3/8;
		for (var i=0;i<rest;i++) {
			elem2 = write_event(event.events[i],sync,bx,by+_height*i/3);
			butt.elem.push(elem2);
			butt.popup.push(elem2.popup);
			all_rect.push(elem2);

		}
		butt.elem.hide();
		
		all_rect.elem = butt;
		//help push the button
		var group = _paper.set();
		arrow1.button =arrow2.button = textn.button = butt.button = butt;
		group.push(arrow1);
		group.push(arrow2);
		group.push(textn);
		group.push(butt);
		group.click(function() {
			animate_button(this.button);
		});
		group.mouseover (function() { this.button.attr("stroke-width",3)});
		group.mouseout (function() {this.button.attr("stroke-width",1)});
		
	}
	return all_rect;

}

/**
	Animate the "more" buttons near filters
**/
function animate_button(butt) {
		butt.cx = butt.cx || 300;
		butt.animate({cx: butt.cx, "stroke-width": butt.cx / 100, fill: butt.cx - 100 ? "hsb(.1, .75, .75)" : "#000", "fill-opacity": +(!!(butt.cx - 100)+0.25)}, 1000, ">");
		butt.cx = butt.cx == 300 ? 100 : 300;
		
		if (butt.activated == 0) {
			butt.elem.toFront();
			butt.elem.show();
			butt.popup.hide();
			butt.activated=1;
		} else {
			butt.elem.hide();
			butt.activated=0;
		}
}

/**
	resets the "more" button
**/
function reset_button(butt) {
	if (butt==0)
		return;
	if (butt.activated==1) {
		butt.cx = butt.cx || 300;
		//butt.animate({cx: butt.cx, "stroke-width": butt.cx / 100, fill: butt.cx - 100 ? "hsb(.1, .75, .75)" : "#000", "fill-opacity": +(!!(butt.cx - 100)+0.25)}, 1000, ">");
		butt.cx = butt.cx == 300 ? 100 : 300;
		butt.attr("fill", "#000");
		butt.attr("fill-opacity",0.3);
		butt.attr("stroke-width",1);
		butt.elem.hide();
		butt.activated=0;
	}

}

/**
	Draw the top-left corner options
**/
function draw_corner() {

	if (get_total_syncpoints_num()!=get_table_height()) { //not all 
		text = _paper.text(80,_yspace/3,"Rows : "+get_start_point() +" to "+get_end_point());
		text.attr("font-size",14);
		text2 = _paper.text(80,_yspace/3+20,"( "+get_total_syncpoints_num()+" )");
		text2.attr("font-size",11);
	}
	if (!_static_view) {		
		plus_start = 10;
		plus_size =11;
		plus_y = 3*_yspace/4;
		plusbutt = _paper.rect(plus_start,plus_y-plus_size/2,plus_size,plus_size);
		path1 = _paper.path("M"+plus_start+" "+plus_y+"L"+(plus_start+plus_size)+" "+plus_y);
		path2=	_paper.path("M"+(plus_start+(plus_size+1)/2)+" "+(plus_y-(plus_size+1)/2)+"L"+(plus_start+(plus_size+1)/2)+" "+(plus_y+(plus_size+1)/2));
		plusbutt.attr("fill", "#fff");
		//makes sure all button is pressable
		plus_set = _paper.set();
		plus_set.push(path1);
		plus_set.push(path2);
		plus_set.push(plusbutt);
		path1.butt=path2.butt=plusbutt.butt = plusbutt;
		plus_set.click(function() {
			for (j=get_start_point();j<=get_end_point();j++) {
				var butt = _collapse_buttons[j];
				if (butt.cond==1)
					collapse(butt);
			}
		});
		
		plus_start=25;
		minus_set = _paper.set();
		minusbutt = _paper.rect(plus_start,plus_y-plus_size/2,plus_size,plus_size);
		path1 = _paper.path("M"+plus_start+" "+plus_y+"L"+(plus_start+plus_size)+" "+plus_y);
		path2=	_paper.path("M"+(plus_start+(plus_size+1)/2)+" "+(plus_y-(plus_size+1)/2)+"L"+(plus_start+(plus_size+1)/2)+" "+(plus_y+(plus_size+1)/2));
		path2.hide();
		minusbutt.attr("fill", "#fff");	
		
		minus_set.push(path1);
		minus_set.push(path2);
		minus_set.push(minusbutt);
		path1.butt=path2.butt=minusbutt.butt = minusbutt;
		minus_set.click(function() {
			for (j=get_start_point();j<=get_end_point();j++) {
				butt = _collapse_buttons[j];
				if (butt.cond==-1)
					collapse(butt);
			}
		});
		minus_set.mouseover (function() { this.butt.attr("stroke-width",3)});
		minus_set.mouseout (function() {this.butt.attr("stroke-width",1)});
		plus_set.mouseover (function() { this.butt.attr("stroke-width",3)});
		plus_set.mouseout (function() {this.butt.attr("stroke-width",1)});	
		
		//refresh button
		ref_butt = _paper.rect(3*_first_width/4-20,3*_yspace/4-8,40,15);
		ref_butt.attr("fill","#ddd");
		ref_butt.butt = ref_butt;
		ref_text = _paper.text(3*_first_width/4,3*_yspace/4,"Reset");
		ref_text.attr("text-size",8);
		ref_text.butt = ref_butt;
		ref_set = _paper.set();
		ref_set.push(ref_text);ref_set.push(ref_butt);
		ref_set.mouseover (function() { this.butt.attr("stroke-width",3)});
		ref_set.mouseout (function() {this.butt.attr("stroke-width",1)});
		ref_set.click (function () { location.reload();
		});
		
		
		//counters = get_num_bthread_type();
		//stat_text = _paper.text(60,_yspace/3-30,"NA: "+counters[0]+",PA:"+counters[1]+",A:"+counters[2]+",L:"+counters[3]);
		if (get_total_syncpoints_num()!=get_table_height()) { //not all 
		//navigation buttons
			left_butt_x = 10;
			butt_size = 18;
			butt_y=_yspace/3;
			var butt_color;
			if (_style==0)
				butt_color = "#aea";
			else
				butt_color= "#eee";
			leftbutt = _paper.rect(left_butt_x,butt_y-butt_size/2,butt_size,butt_size);
			leftbutt.attr("fill",butt_color);
			rightbutt = _paper.rect(_first_width-left_butt_x-3*butt_size/2,butt_y-butt_size/2,butt_size,butt_size);
			rightbutt.attr("fill",butt_color);
			leftbutt.enable = (get_start_point()!=1);
			rightbutt.enable = (get_end_point()!=get_total_syncpoints_num());
			if (!leftbutt.enable) leftbutt.attr("fill","#aaa");
			if (!rightbutt.enable) rightbutt.attr("fill","#aaa");	
			arrow1 = _paper.path("M-5 5L0 0L5 5");
			arrow1.translate(left_butt_x+butt_size/2,butt_y-3);
			arrow1.attr("stroke","#000");
			arrow1.butt = leftbutt;
			
			arrow2 = _paper.path("M-5 -5L0 0L5 -5");
			arrow2.translate(_first_width-left_butt_x-butt_size,butt_y+3);
			arrow2.attr("stroke","#000");
			arrow2.butt = rightbutt;
			
			//setting the navigation buttons functions
			leftbutt.click(function() {click_navigate_left(this);});
			rightbutt.click(function() {click_navigate_right(this);});
			arrow1.click(function() { click_navigate_left(this.butt)});
			arrow2.click(function() { click_navigate_right(this.butt)});
			leftbutt.mouseover (function() {if (this.enable) this.attr("stroke-width",3)});
			leftbutt.mouseout (function() {if (this.enable) this.attr("stroke-width",1)});
			rightbutt.mouseover (function() {if (this.enable) this.attr("stroke-width",3)});
			rightbutt.mouseout (function() {if (this.enable) this.attr("stroke-width",1)});
			arrow1.mouseover (function() {if (this.butt.enable) this.butt.attr("stroke-width",3)});
			arrow1.mouseout (function() {if (this.butt.enable) this.butt.attr("stroke-width",1)});
			arrow2.mouseover (function() {if (this.butt.enable) this.butt.attr("stroke-width",3)});
			arrow2.mouseout (function() {if (this.butt.enable) this.butt.attr("stroke-width",1)});
		}
	}
}

/**
	Navigate forward in the syncpoints
**/
function click_navigate_right (butt) {
		if (butt.enable==true) {
			var first = get_start_point() +get_sync_num_in_page();
			max_first = get_total_syncpoints_num()-get_sync_num_in_page()+1;
			if (first>=max_first) { first=max_first;}
			constrain_run(first);
			_paper.clear();
			_paper.remove();
			draw_bpj_table();
		}
}
/**
	Navigate backward in the syncpoints
**/
function click_navigate_left(butt) {
		if (butt.enable==true) {
			var first = get_start_point() -get_sync_num_in_page();
			if (first<=1) {first=1;}
			constrain_run(first);
			_paper.clear();
			_paper.remove();
			draw_bpj_table();
		}
}

/**
	Scroll to the next leader (using shift)
**/
function scroll_to_next_leader() {
	_chosen_row++;
	if (_chosen_row > get_end_point() || _chosen_row < get_start_point())
		_chosen_row = get_start_point();
	
	y= _chosen_row-get_start_point()+1;
	x = get_leader_in_row(_chosen_row);
	if (x==0) return 0;
	extra=0;
	for (var k=get_start_point();k<_chosen_row;k++) {
		if (_collapse_buttons[k].cond!=1)
			extra= extra+_sizes[k]-3;
		else
			extra = -2;
	}
		
	butt.cond;
	window.scroll(_xspace*(x-1),_yspace*(y-1)+extra*_height/3);
}

/**
	Draw a syncpoint row
**/
function draw_bpj_row_line(sync) {

	all_rects = _paper.set();
	yind = sync.index;
	all_rects.icons = _paper.set(); //all the icons (currently only state)
	all_rects.iconsmall = _paper.set(); //small version icons
	all_rects.iconspop = _paper.set(); //the icons popups
	
	extra=0;
	for (var k=get_start_point();k<yind;k++)
		extra= extra+_sizes[k]-3;
	extra=extra*_height/3;
	y2=yind*_yspace-_ymar/2;
	x = _xspace*(_nx)+_first_width;
	long_path = _paper.path("M5 "+y2+"L"+x+" "+y2);
	all_rects.push(long_path);

	//y2=(yind+1)*_yspace-_ymar/2;
	//path = _paper.path("M5 "+y2+"L"+x+" "+y2);
	//all_rects.push(path);
	max_text = Math.floor(_first_width/10);
	name = sync.event.name;
	popup = 0;
	if (name!=undefined) {
		if( name.length>max_text)
		{
			name = name.substr(0,max_text-2);
			name = name.concat("...");
			popup=1;
		}
		text =_paper.text(_first_width/2+11,yind*_yspace+_height/2,name);
		text.attr("font-weight",'bolder');
		text.attr("font-size",15);
	if (popup==1)  {
		popuper = register_popup(_first_width/2,yind*_yspace+_height/2,text,sync.event.name,1);
		all_rects.push(popuper);
		all_rects.popup = popuper;
	} else 
		all_rects.popup=_paper.set();
		all_rects.push(text);
	}

	text2 =_paper.text(34,yind*_yspace+_height/2,yind);
	all_rects.push(text2);
	text2.attr("font-size",15);
	//plus and minus collapse
	if (!_static_view) {
		plus_start = 10;
		plus_size =11;
		plus_y = yind*_yspace+_height/2;
		butt = _paper.rect(plus_start,plus_y-plus_size/2,plus_size,plus_size);
		path1 = _paper.path("M"+plus_start+" "+plus_y+"L"+(plus_start+plus_size)+" "+plus_y);
		path2=	_paper.path("M"+(plus_start+(plus_size+1)/2)+" "+(plus_y-(plus_size+1)/2)+"L"+(plus_start+(plus_size+1)/2)+" "+(plus_y+(plus_size+1)/2));
		path2.hide();
		butt.index = yind;
		butt.path2 = path2;
		butt.longpath = long_path;
		butt.attr("fill", "#fff");
		butt.cond = -1;
		all_rects.push(butt);
		all_rects.push(path1);
		all_rects.push(path2);
		butt.butt = butt;
		text.butt=text2.butt=path1.butt = butt;
		path2.butt = butt;
		_collapse_buttons[yind]= butt;
		all_rects.click(function() {
			if (this.butt!=undefined) collapse(this.butt);
		});
		all_rects.mouseover (function()  {if (this.butt!=undefined) this.butt.attr("stroke-width",3)});
		all_rects.mouseout (function() {if (this.butt!=undefined) this.butt.attr("stroke-width",1)});

	}
	
	//state icon
	if (sync.state.length!=0) {
	//normal icon
	/*
		ipath = "M15.067,2.25c-5.979,0-11.035,3.91-12.778,9.309h3.213c1.602-3.705,5.271-6.301,9.565-6.309c5.764,0.01,10.428,4.674,10.437,10.437c-0.009,5.764-4.673,10.428-10.437,10.438c-4.294-0.007-7.964-2.605-9.566-6.311H2.289c1.744,5.399,6.799,9.31,12.779,9.312c7.419-0.002,13.437-6.016,13.438-13.438C28.504,8.265,22.486,2.252,15.067,2.25zM10.918,19.813l7.15-4.126l-7.15-4.129v2.297H-0.057v3.661h10.975V19.813z"
		icon = _paper.path(ipath);
		icon.translate(_first_width-40,y2+5);

		icon.scale(0.7,0.7);
		if (sync.is_state_modified())
			icon.attr("fill","#226");
		else {
			icon.attr("fill","#226");
			icon.attr("stroke-opacity",0.5);
			icon.attr("opacity",0.5);
			}
		var circle_but = _paper.circle(_first_width-25,y2+20,11);
		circle_but.attr("fill","#222");
		circle_but.attr("opacity",0);
		circle_but.icon = icon;
		popup = register_popup(_first_width+80,y2,circle_but,sync.display_state(),sync.state.length);
		circle_but.mouseover (function()  {this.icon.attr("stroke-width",2)});
		circle_but.mouseout (function()  {this.icon.attr("stroke-width",1)});
		all_rects.icons.push(icon);
		all_rects.icons.push(circle_but);
		all_rects.iconspop.push(popup);
		all_rects.push(all_rects.icons);
		all_rects.push(popup);
		*/
		//state presentation icon
		total_size = _first_width/2-20;
		icon_y = y2+5;
		icon_x = _first_width/2;
		icon_height = 20;
		var num; var num2;
		num = Math.sqrt(sync.state.length);
		if (num<=7 && num == Math.floor(Math.sqrt(sync.state.length))) //small square {
			num2=num;
		else {
			num2=5; num = Math.ceil(sync.state.length/num2);
		}
		for (var i=0;i<sync.state.length;i++) {
			 //only works for square state
				pos1 = i%num;
				pos2 = Math.floor(i/num);
				delta = total_size/num;
				delta_y = icon_height/num2;
				small_box = _paper.rect(icon_x+pos1*delta,icon_y+pos2*delta_y,delta,delta_y);
				smaller_box = _paper.rect(icon_x+pos1*delta/2+30,icon_y+pos2*delta_y/2+10,delta/2,delta_y/2);
			var color
			switch(sync.state[i].status) {

				case 0: color = "#ddd"; break;
				case 1: color = "#f54"; break;
				case 2: color = "#3af"; break;
				case 3: color = "#0ff"; break;
				case 4: color = "#ff0"; break;
				case 5: color = "#f0f"; break;
				case 6: color= "#777"; break;
				case 7: color="#28a"; break;
				case 8: color="#7f4"; break;
				default: color = "#fff"; break;

			}
			small_box.attr("fill",color);
			smaller_box.attr("fill",color);
			all_rects.icons.push(small_box);
			all_rects.iconsmall.push(smaller_box);
		}
		var big_box = _paper.rect(icon_x,icon_y,total_size,icon_height);
		all_rects.icons.push(big_box);
		if (sync.is_state_modified()) {
			big_box.attr("stroke","#000");
			}
		else
			big_box.attr("stroke","#aaa");
		
		big_box2=_paper.rect(icon_x+30,icon_y+10,total_size/2,icon_height/2);
		all_rects.iconsmall.push(big_box2);
		if (sync.is_state_modified()) {
			big_box2.attr("stroke","#000");
			}
		else
			big_box2.attr("stroke","#aaa");
		
		box_bp= _paper.rect(icon_x,icon_y,total_size,icon_height);
		box_bp.attr("fill","#000");
		box_bp.attr("opacity",0);
		box_bp.butt=big_box;
		all_rects.icons.push(box_bp);
		
		box_bp2= _paper.rect(icon_x+30,icon_y+10,total_size/2,icon_height/2);
		box_bp2.attr("fill","#000");
		box_bp2.attr("opacity",0);
		box_bp2.butt=big_box2;
		all_rects.iconsmall.push(box_bp2);
		
		popup = register_popup(_first_width+80,y2,box_bp,sync.display_state(),sync.state.length);
		popup2 = register_popup(_first_width+80,y2,box_bp2,sync.display_state(),sync.state.length);
		box_bp.mouseover (function()  {this.butt.attr("stroke-width",2)});
		box_bp.mouseout (function()  {this.butt.attr("stroke-width",1)});
		box_bp2.mouseover (function()  {this.butt.attr("stroke-width",2)});
		box_bp2.mouseout (function()  {this.butt.attr("stroke-width",1)});

		all_rects.push(all_rects.icons);
		all_rects.push(all_rects.iconsmall);
		all_rects.iconsmall.hide();
		all_rects.push(popup);
		all_rects.push(popup2);
		
		
	}
	all_rects.translate(0,extra-(get_start_point()-1)*_yspace);
	return all_rects;
}

/**
	collapse a row corresponding to the given button
**/
function collapse (butt) {
	extra = _height*(_sizes[butt.index]-3)/3;
	for (var i=butt.index+1;i<=get_end_point();i++)
		for (var j=0;j<=_nx;j++)
			if (_rects_array[i][j]!=undefined && _rects_array[i][j].length!=0)
				_rects_array[i][j].translate(0,butt.cond*((_yspace)*5/8+extra));
	for (var j=1;j<=_nx;j++) //only the current line
	{
		if (_rects_array[butt.index][j]==undefined || _rects_array[butt.index][j].length==0)
			continue;
			
		
		if (butt.cond==-1)
		{
			_rects_array[butt.index][j].hide();

			butt.path2.show();
			}
		else
		{
			_rects_array[butt.index][j].show();
			for (var m=0;m<_rects_array[butt.index][j].elem.length;m++)
			{
//					alert(m);
				reset_button(_rects_array[butt.index][j].elem[m]);
				_rects_array[butt.index][j].elem[m].elem.hide();
				}
			butt.path2.hide();
		}
		var total = _rects_array[butt.index][j].total;
		//taking care of the learning extension icons
		_rects_array[butt.index][j].learningre.show(); 
		_rects_array[butt.index][j].learningstate.show(); 
		_rects_array[butt.index][j].learningre.translate(0,butt.cond*(_yspace*5/8+ (total-3)*_height/3+(_height/6)*(_sizes[butt.index]-total)));
		_rects_array[butt.index][j].learningre.translate(0,butt.cond*(-5)); //fix because of the shorter place
		_rects_array[butt.index][j].learningstate.translate(0,butt.cond*(-5));
		rect = _rects_array[butt.index][j].box;
		rect.show();
		var total = _rects_array[butt.index][j].total;
		rect.animate({height: (total/3+1/4)*_height/2+butt.cond*(total/3-1/4)*_height/2}, 700, "elastic");
		
		line = _rects_array[butt.index][j].line;
		if (line!=0) {
			scalar = ((_sizes[butt.index]-total)*_height/3+(2*_ymar))/(2*_ymar);
	//		alert(butt.cond==1 ? scalar : 1/scalar);
			if (butt.cond==1) {
				line.scale(1,1);
				line.translate(0,(_yspace*5/8+ (total-3)*_height/3+(_height/6)*(_sizes[butt.index]-total)));
			}
			else {
				trans= -1*(_yspace*5/8+ (total-3)*_height/3);
				line.scale(1,1/scalar);
				line.translate(0,-1*(_yspace*5/8+ (total-3)*_height/3+(_height/6)*(_sizes[butt.index]-total)));
			}
			line.show();
			}
	}
	for (var j=0;j<=_nx;j++)  {//hide popups
		if (_rects_array[butt.index][j]!=undefined && _rects_array[butt.index][j].popup!=undefined)
			for (var m=0;m<_rects_array[butt.index][j].popup.length;m++)	 {
				_rects_array[butt.index][j].popup[m].hide();
		}
	}
	//the row header
	if (_rects_array[butt.index][0].length!=0) {
		_rects_array[butt.index][0].translate(0,_height/3*butt.cond); 
		_rects_array[butt.index][0].icons.translate(0,(-7-_height/6)*butt.cond);
		_rects_array[butt.index][0].iconsmall.translate(0,(-7-_height/6)*butt.cond);
		_rects_array[butt.index][0].iconspop.translate(0,(-7-_height/6)*butt.cond);
		if (butt.cond==-1) {
			_rects_array[butt.index][0].icons.scale(0.5,0.5);
			_rects_array[butt.index][0].iconsmall.show();
			_rects_array[butt.index][0].icons.hide();
		} else {
			_rects_array[butt.index][0].icons.scale(1,1);
			_rects_array[butt.index][0].iconsmall.hide();
			_rects_array[butt.index][0].icons.show();
		}
	}
		
	butt.longpath.translate(0,-_height/3*butt.cond); //keep the long line in place
	butt.cond=-1*butt.cond;

}

/**
	Draw a bthread column lines and header
**/
function draw_bpj_col_line(bth,extra,xind) {
	x2=(xind-1)*_xspace+_first_width-_xmar/2;
	y2 = _yspace*(1+_ny)+extra*_height/3;
	path=_paper.path("M"+x2+" 5L"+x2+" "+y2);
	try {
	path.attr("stroke-opacity",0.5);
	} catch(ex) {}
	x2=x2+_xspace;
	path=_paper.path("M"+x2+" 5L"+x2+" "+y2);
	try {
	path.attr("stroke-opacity",0.5);
	} catch(e) {}
	x2= x2-_width/2-_xmar/2;
	num_text =_paper.text(x2,15,xind);
		max_text = Math.floor(_width/8);
	name = bth.name;
	popup=0;
	if (name.length>max_text)
	{
		name = name.substr(0,max_text-2);
		name = name.concat("...");
		popup=1;
	}
	text =_paper.text(x2,40,name);
	if (popup==1) 
		register_popup(x2,_yspace/2,text,bth.name,1);
	text.attr("font-size",16);
	
	
	y2= _yspace*(_ny);
	
	
	//group
	
	var text2;
	if (bth.type==0)
		text2='Non-Active';
	else if(bth.type==1)
		text2='Follower';
	else if(bth.type==2)
		text2='Active';
	else
		text2='Leader';
	subtext =_paper.text(x2,70,text2);
	
	//mid-navigate buttons
	if (!_static_view && get_total_syncpoints_num()!=get_table_height()) {
		butt_x = _width/3.3;
		butt_size = 16;
		butt_y=70;
		var butt_color;
		if (_style==0)
			butt_color = "#aea";
		else 
			butt_color= "#eee";
		leftbutt = _paper.rect(x2-butt_x-butt_size/2,butt_y-butt_size/2,butt_size,butt_size);
		leftbutt.attr("fill",butt_color);
		rightbutt = _paper.rect(x2+butt_x-butt_size/2,butt_y-butt_size/2,butt_size,butt_size);
		rightbutt.attr("fill",butt_color);
		//leftbutt.enable = (get_start_point()!=1);
		//rightbutt.enable = (get_end_point()!=get_total_syncpoints_num());
		leftbutt.enable=rightbutt.enable=false;
		next = get_end_point()+1;
		if (bth.nextbsync[next]<get_total_syncpoints_num()+1)
		{
			rightbutt.enable=true;
			rightbutttext = _paper.text(x2+butt_x+25,butt_y,bth.nextbsync[next]);
			rightbutt.sync = bth.nextbsync[next];
		}
		prev = get_start_point()-1;
		if (bth.prevbsync[prev]>0)
		{
			leftbutt.enable=true;
			leftbutttext = _paper.text(x2-butt_x-25,butt_y,bth.prevbsync[prev]);
			leftbutt.sync=bth.prevbsync[prev];
		}
		
		
		if (!leftbutt.enable) leftbutt.attr("fill","#aaa");
		if (!rightbutt.enable) rightbutt.attr("fill","#aaa");	
		
		arrow1 = _paper.path("M-5 5L0 0L5 5");
		arrow1.translate(x2-butt_x-butt_size/2+butt_size/2,butt_y-3);
		arrow1.attr("stroke","#000");
		arrow1.butt = leftbutt;
		
		arrow2 = _paper.path("M-5 -5L0 0L5 -5");
		arrow2.translate(x2+butt_x-butt_size/2+butt_size/2,butt_y+3);
		arrow2.attr("stroke","#000");
		arrow2.butt = rightbutt;
		
		
		leftbutt.click(function() {click_navigate_bth(this);});
		rightbutt.click(function() {click_navigate_bth(this);});
		arrow1.click(function() { click_navigate_bth(this.butt)});
		arrow2.click(function() { click_navigate_bth(this.butt)});
		leftbutt.mouseover (function() {if (this.enable) this.attr("stroke-width",3)});
		leftbutt.mouseout (function() {if (this.enable) this.attr("stroke-width",1)});
		rightbutt.mouseover (function() {if (this.enable) this.attr("stroke-width",3)});
		rightbutt.mouseout (function() {if (this.enable) this.attr("stroke-width",1)});
		arrow1.mouseover (function() {if (this.butt.enable) this.butt.attr("stroke-width",3)});
		arrow1.mouseout (function() {if (this.butt.enable) this.butt.attr("stroke-width",1)});
		arrow2.mouseover (function() {if (this.butt.enable) this.butt.attr("stroke-width",3)});
		arrow2.mouseout (function() {if (this.butt.enable) this.butt.attr("stroke-width",1)});
	}
}

/**
	Navigate to another syncpoint of a bthread
**/
function click_navigate_bth (butt) {
		if (butt.enable==true) {
			var first = butt.sync -Math.ceil(get_sync_num_in_page()/2)+1; //puts it in the middle
			if (first<1) first=1;
			constrain_run(first);
			_paper.clear();
			_paper.remove();
			draw_bpj_table();
		}
}


/**
	Main drawing function, should be called once
**/
function draw_bpj_table() {
	_nx=get_table_width();
	_ny=get_table_height();
	_rects_array = new Array(_ny+1);
	_sizes= new Array(_ny+1);
	startpoint = get_start_point();
	endpoint = get_end_point();
		//setting _sizes array
	extra=0;
	all_bthreads = get_filtered_bthread_list();
	for (j=startpoint;j<=endpoint;j++) {
		var sync = get_syncpoint(j);
		for (i=0;i<all_bthreads.length;i++) {
			var bthr = all_bthreads[i];
			update_sizes(sync,bthr);
		}
		extra = extra+_sizes[j]-3;
	}
	_paper = Raphael(10, 10, _xspace*(all_bthreads.length+3),_yspace*(_ny+3)+extra*_height/3);
	for (i=0;i<all_bthreads.length;i++)
	{
		var bthr = all_bthreads[i];
		draw_bpj_col_line(bthr, extra, i+1); //extra is the extra y space needed
	}
	_collapse_buttons = new Array(_ny+1);
	draw_corner();
	for (var j=startpoint;j<=endpoint;j++) { //draw horizontal lines
		_rects_array[j]=new Array(_nx+1);
		var sync = get_syncpoint(j);
		_rects_array[j][0]=draw_bpj_row_line(sync);
	}
	for (var j=startpoint;j<=endpoint;j++)
	{
		var sync = get_syncpoint(j);
		for (i=0;i<all_bthreads.length;i++)
		{
			var bthr = all_bthreads[i];
			_rects_array[j][bthr.index] = draw_bpj_rect(sync,bthr,i+1);
		}
	}
	return _paper;
}